import re
from click import (
    Option,
    UsageError,
    ParamType
)
from datetime import date


class MutuallyExclusiveAndRequireOption(Option):
    """
    Mutually exclude other specified options

    Note: this implementation refers to https://gist.github.com/jacobtolar/fb80d5552a9a9dfc32b12a829fa21c0c
    """

    def __init__(self, *args, **kwargs):
        self.mutually_exclusive = set(kwargs.pop('mutually_exclusive', []))
        self.requires = set(kwargs.pop('requires', []))
        help = kwargs.get('help', '')
        if self.mutually_exclusive:
            ex_str = ', '.join(self.mutually_exclusive)
            kwargs['help'] = help + (
                ' NOTE: This argument is mutually exclusive with '
                ' arguments: [' + ex_str + '].'
            )
        super(MutuallyExclusiveAndRequireOption, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        if self.name in opts and self.mutually_exclusive and self.mutually_exclusive.intersection(opts):
            raise UsageError(
                "Illegal usage: `{}` is mutually exclusive with "
                "arguments `{}`.".format(
                    self.name,
                    ', '.join(self.mutually_exclusive)
                )
            )

        if self.name in opts and self.requires and not self.requires.intersection(opts):
            raise UsageError(
                "Illegal usage: `{}` requires "
                "arguments `{}`.".format(
                    self.name,
                    ', '.join(self.requires)
                )
            )

        return super(MutuallyExclusiveAndRequireOption, self).handle_parse_result(
            ctx,
            opts,
            args
        )


class DateStrParamType(ParamType):
    name = 'DateString'

    def convert(self, value, param, ctx):
        try:
            if len(value) != 8:
                raise ValueError('invalid date string length')
            year, month, day = value[:4], value[4:6], value[6:]
            date(int(year), int(month), int(day))
            return value
        except (TypeError, ValueError):
            self.fail('{} is not a valid date string, date string should be YYYYMMDD'.format(value), param, ctx)


class MetadataParamType(ParamType):
    name = 'MetadataString'

    def convert(self, value, param, ctx):
        try:
            m = re.match('([-\w]*\w):([-\w]*\w+)', value)
            if not m:
                self.fail('{} is not a valid metadata, metadata should be "fieldname:fieldvalue"'.format(value),
                          param, ctx)
            return m.group(1, 2)
        except TypeError:
            self.fail('{} is not a valid metadata, metadata should be "fieldname:fieldvalue"'.format(value), param, ctx)


class EnvParamType(ParamType):
    name = 'EnvironmentString'

    def convert(self, value, param, ctx):
        try:
            m = re.match('([-\w]*\w):([-\w]*\w+)', value)
            if not m:
                self.fail('{} is not a valid environment variable, environment variable should be "Key:Val"'
                          .format(value), param, ctx)
                return
            return m.group(1, 2)
        except TypeError:
            self.fail('{} is not a valid environment variable, environment variable should be "Key:Val"'.format(value),
                      param, ctx)


class UserParamType(ParamType):
    name = 'UserParamString'

    def convert(self, value, param, ctx):
        try:
            m = re.match('([-\w]*\w):([-\w]*\w+)', value)
            if not m:
                self.fail('{} is not a valid user parameter, user parameter should be "Key:Val"'
                          .format(value), param, ctx)
                return
            # NOTE: m[1], m[2] is more clear, but `match.__getitem__(g)` is available from version 3.6
            # https://docs.python.org/3.6/library/re.html#re.match.__getitem__
            return m.group(1, 2)
        except TypeError:
            self.fail('{} is not a valid user parameter, valid user parameter should be "Key:Val"'.format(value),
                      param, ctx)


DATE_STR = DateStrParamType()
METADATA_STR = MetadataParamType()
ENVIRONMENT_STR = EnvParamType()
USER_PARAM_STR = UserParamType()
